@@ -25,7 +25,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -60,7 +60,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -92,13 +92,13 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: Target.cc 7145 2008-04-11 03:54:44Z david $ */
+/* $Id: Target.cc 7640 2008-05-22 20:45:32Z fyodor $ */
 
 #ifdef WIN32
 #include "nmap_winconfig.h"
@@ -139,6 +139,7 @@ void Target::Initialize() {
   MACaddress_set = SrcMACaddress_set = NextHopMACaddress_set = false;
   htn.msecs_used = 0;
   htn.toclock_running = false;
+  htn.host_start = htn.host_end = 0;
   interface_type = devt_other;
 	devname[0] = '\0';
 	devfullname[0] = '\0';
@@ -389,6 +390,7 @@ void Target::startTimeOutClock(const struct timeval *now) {
   htn.toclock_running = true;
   if (now) htn.toclock_start = *now;
   else gettimeofday(&htn.toclock_start, NULL);
+  if (!htn.host_start) htn.host_start = htn.toclock_start.tv_sec;
 }
   /* The complement to startTimeOutClock. */
 void Target::stopTimeOutClock(const struct timeval *now) {
@@ -398,6 +400,7 @@ void Target::stopTimeOutClock(const struct timeval *now) {
   if (now) tv = *now;
   else gettimeofday(&tv, NULL);
   htn.msecs_used += TIMEVAL_MSEC_SUBTRACT(tv, htn.toclock_start);
+  htn.host_end = tv.tv_sec;
 }
   /* Returns whether the host is timedout.  If the timeoutclock is
      running, counts elapsed time for that.  Pass NULL if you don't have the
@@ -473,3 +476,4 @@ void Target::osscanSetFlag(int flag) {
 		osscan_flag = flag;
 }
 
+
