@@ -3,21 +3,23 @@
  * nbase.h -- The main include file exposing the external API for          *
  * libnbase, a library of base (often compatability) routines.  Programs   *
  * using libnbase can guarantee the availability of functions like         *
- * (v)snprintf and inet_aton.  This library also provides consistency and  *
+ * (v)snprintf and inet_pton.  This library also provides consistency and  *
  * extended features for some functions.  It was originally written for    *
- * use in the Nmap Security Scanner ( http://www.insecure.org/nmap/ ).     *
+ * use in the Nmap Security Scanner ( http://nmap.org ).                   *
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1995-2003 Insecure.Com LLC. This       *
- * program is free software; you may redistribute and/or modify it under   *
- * the terms of the GNU General Public License as published by the Free    *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as   *
- * our remote OS fingerprinting database and code.                         *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -26,9 +28,12 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
- * o Executes Nmap                                                         *
- * o Integrates/includes/aggregates Nmap into an executable installer      *
+ *   nmap-os-db or nmap-service-probes.                                    *
+ * o Executes Nmap and parses the results (as opposed to typical shell or  *
+ *   execution-menu apps, which simply display raw Nmap output and so are  *
+ *   not derivative works.)                                                * 
+ * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+ *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
@@ -37,7 +42,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://nmap.org to download Nmap.                                       *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -49,14 +54,23 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         *
  * If you received these files with a written license agreement or         *
- * contract stating terms other than the (GPL) terms above, then that      *
+ * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
@@ -67,7 +81,7 @@
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
+ * distribution.  By sending these changes to Fyodor or one of the         *
  * Insecure.Org development mailing lists, it is assumed that you are      *
  * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
  * to reuse, modify, and relicense the code.  Nmap will always be          *
@@ -81,12 +95,13 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html .                                  *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nbase.h,v 1.32 2003/09/22 09:10:12 fyodor Exp $ */
+/* $Id: nbase.h 7641 2008-05-22 20:45:49Z fyodor $ */
 
 #ifndef NBASE_H
 #define NBASE_H
@@ -97,7 +112,7 @@
  * * 'inline' is defined to what is neccessary for the C compiler being
  *   used (which may be nothing)
  *
- * * snprintf, inet_aton, inet_pton, inet_aton, memcpy, and bzero are 
+ * * snprintf, inet_pton, memcpy, and bzero are 
  *   provided if you don't have them (prototypes for these are 
  *   included either way).
  *
@@ -107,7 +122,7 @@
  *   will generally be one of the following: LINUX, FREEBSD, NETBSD,
  *   OPENBSD, SOLARIS, SUNOS, BSDI, IRIX, NETBSD
  *
- * * Insures that GNU getopt_* functions exist (such as getopt_long_only
+ * * Insures that getopt_* functions exist (such as getopt_long_only)
  *
  * * Various string functions such as Strncpy() and strcasestr() see protos 
  *   for more info.
@@ -165,11 +180,14 @@
 #endif
 
 #ifndef HAVE___ATTRIBUTE__
-#define __attribute__(x)
+#define __attribute__(args)
 #endif
 
 #include <stdarg.h>
 
+/* Keep assert() defined for security reasons */
+#undef NDEBUG
+
 /* Integer widths */
 #if (SIZEOF_CHAR == 1)
 typedef unsigned char u8;
@@ -255,17 +273,12 @@ extern "C" int snprintf (char *, size_t, const char *, ...);
 extern "C" int vsnprintf (char *, size_t, const char *, va_list);
 #endif
 
-/* GNU getopt replacements ... Anyone have a BSD licensed version of these? */
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #else
-/* The next half-dozen lines are from gcc-2.95 ... -Fyodor */
-/* Include getopt.h for the sake of getopt_long.
-   We don't need the declaration of getopt, and it could conflict
-   with something from a system header file, so effectively nullify that.  */
-#define getopt getopt_loser
+#ifndef HAVE_GETOPT_LONG_ONLY
 #include "getopt.h"
-#undef getopt
+#endif
 #endif /* HAVE_GETOPT_H */
 
 /* More Windows-specific stuff */
@@ -278,13 +291,37 @@ extern "C" int vsnprintf (char *, size_t, const char *, va_list);
 #define S_ISDIR(m)      (((m) & _S_IFMT) == _S_IFDIR)
 #endif
 
-#define stat _stat // wtf was ms thinking?
+/* Windows doesn't have the access() defines */
+#ifndef F_OK
+#define F_OK 00
+#endif
+#ifndef W_OK
+#define W_OK 02
+#endif
+#ifndef R_OK
+#define R_OK 04
+#endif
+
+#define access _access
+#define stat _stat /* wtf was ms thinking? */
 #define execve _execve
+#define getpid _getpid
+#define dup _dup
+#define dup2 _dup2
+#define strdup _strdup
+#define write _write
+#define open _open
+#define stricmp _stricmp
+#define putenv _putenv
+
+#if !defined(__GNUC__)
 #define snprintf _snprintf
 #define vsnprintf _vsnprintf
+#endif
+
 #define strcasecmp _stricmp
 #define strncasecmp _strnicmp
-
+#define execv _execv
 
 #endif /* WIN32 */
 
@@ -301,9 +338,14 @@ extern "C" int vsnprintf (char *, size_t, const char *, va_list);
 extern "C" {
 #endif
 
-// Returns the UNIX/Windows errno-equivalent.  Note that the Windows call is socket/networking specific.  The windows error number
-// returned is like WSAMSGSIZE, but nbase.h includes #defines to correlate many of the common UNIX errors with their closest 
-// Windows equivalents.  So you can use EMSGSIZE or EINTR.
+  /* Returns the UNIX/Windows errno-equivalent.  Note that the Windows
+     call is socket/networking specific.  Also, WINDOWS TENDS TO RESET
+     THE ERROR, so it will return success the next time.  So SAVE THE
+     RESULTS and re-use them, don't keep calling socket_errno().  The
+     windows error number returned is like WSAMSGSIZE, but nbase.h
+     includes #defines to correlate many of the common UNIX errors
+     with their closest Windows equivalents.  So you can use EMSGSIZE
+     or EINTR. */
 int socket_errno();
 
 /* The usleep() function is important as well */
@@ -337,21 +379,32 @@ int gettimeofday(struct timeval *tv, struct timeval *tz);
 unsigned int sleep(unsigned int seconds);
 #endif
 
-#ifndef HAVE_INET_ATON
-struct in_addr;
-int inet_aton(const char *cp, struct in_addr *addr);
-#endif
-
 /* Strncpy is like strcpy() except it ALWAYS zero-terminates, even if
    it must truncate */
 int Strncpy(char *dest, const char *src, size_t n);
 
+int Vsnprintf(char *, size_t, const char *, va_list)
+     __attribute__ ((format (printf, 3, 0)));
+int Snprintf(char *, size_t, const char *, ...)
+     __attribute__ ((format (printf, 3, 4)));
+
+/* Trivial function that returns nonzero if all characters in str of
+   length strlength are printable (as defined by isprint()) */
+int stringisprintable(const char *str, int strlength);
+
+/* Convert non-printable characters to replchar in the string */
+void replacenonprintable(char *str, int strlength, char replchar);
+
+/* Portable, incompatible replacements for dirname and basename. */
+char *path_get_dirname(const char *path);
+char *path_get_basename(const char *path);
+
 /* A few simple wrappers for the most common memory allocation routines which will exit() if the
 	allocation fails, so you don't always have to check -- see nbase_memalloc.c */
-void *safe_malloc(int size);
+void *safe_malloc(size_t size);
 void *safe_realloc(void *ptr, size_t size);
 /* Zero-initializing version of safe_malloc */
-void *safe_zalloc(int size);
+void *safe_zalloc(size_t size);
 
   /* Some routines for obtaining simple (not secure on systems that
      lack /dev/random and friends' "random" numbers */
@@ -363,6 +416,8 @@ u32 get_random_u32();
 u16 get_random_u16();
 u8 get_random_u8();
 
+/* CRC16 Cyclic Redundancy Check  */
+unsigned long crc16(unsigned char *buf, int len);
 
 #ifndef STDIN_FILENO
 #define STDIN_FILENO 0
