@@ -4,17 +4,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -23,7 +23,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -37,7 +37,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://nmap.org to download Nmap.                                       *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -49,16 +49,16 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -76,7 +76,7 @@
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
+ * distribution.  By sending these changes to Fyodor or one of the         *
  * Insecure.Org development mailing lists, it is assumed that you are      *
  * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
  * to reuse, modify, and relicense the code.  Nmap will always be          *
@@ -90,9 +90,9 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
@@ -142,6 +142,9 @@
 // doug at hcsw.org
 // http://www.hcsw.org
 
+/*
+ * DNS Caching and ageing added by Eddie Bell ejlbell@gmail.com 2007
+ */
 
 // TODO:
 //
@@ -158,6 +161,7 @@
 #include <limits.h>
 #include <list>
 #include <vector>
+#include <algorithm>
 
 #include "nmap.h"
 #include "NmapOps.h"
@@ -165,6 +169,8 @@
 #include "nsock.h"
 #include "utils.h"
 #include "nmap_tty.h"
+#include "timing.h"
+#include "Target.h"
 
 extern NmapOps o;
 
@@ -234,6 +240,8 @@ static int read_timeouts[][4] = {
 // Size of hash table used to hold the hosts from /etc/hosts
 #define HASH_TABLE_SIZE 256
 
+// Hash macro for etchosts
+#define IP_HASH(x) (ntohl(x)%HASH_TABLE_SIZE)
 
 
 //------------------- Internal Structures ---------------------
@@ -267,6 +275,7 @@ struct request_s {
 struct host_elem_s {
   char *name;
   u32 addr;
+  u8 cache_hits;
 };
 
 
@@ -278,6 +287,7 @@ static std::list<request *> cname_reqs;
 static int total_reqs;
 static nsock_pool dnspool=NULL;
 
+/* The DNS cache, not just for entries from /etc/hosts. */
 static std::list<host_elem *> etchosts[HASH_TABLE_SIZE];
 
 static int stat_actual, stat_ok, stat_nx, stat_sf, stat_trans, stat_dropped, stat_cname;
@@ -285,19 +295,20 @@ static struct timeval starttv;
 static int read_timeout_index;
 static u16 id_counter;
 
+static int firstrun=1;
 static ScanProgressMeter *SPM;
 
 
 //------------------- Prototypes and macros ---------------------
 
-void put_dns_packet_on_wire(request *req);
+static void put_dns_packet_on_wire(request *req);
+static char *lookup_etchosts(u32 ip);
+static void addto_etchosts(u32 ip, const char *hname);
 
 #define ACTION_FINISHED 0
 #define ACTION_CNAME_LIST 1
 #define ACTION_TIMEOUT 2
 
-
-
 //------------------- Misc code --------------------- 
 
 static void output_summary() {
@@ -369,7 +380,8 @@ static void do_possible_writes() {
       }
 
       if (tpreq) {
-        if (o.debugging >= TRACE_DEBUG_LEVEL) log_write(LOG_STDOUT, "mass_rdns: TRANSMITTING for <%s> (server <%s>)\n", tpreq->targ->targetipstr() , tpserv->hostname);
+        if (o.debugging >= TRACE_DEBUG_LEVEL)
+	   log_write(LOG_STDOUT, "mass_rdns: TRANSMITTING for <%s> (server <%s>)\n", tpreq->targ->targetipstr() , tpserv->hostname);
         stat_trans++;
         put_dns_packet_on_wire(tpreq);
       }
@@ -390,14 +402,13 @@ static void write_evt_handler(nsock_pool nsp, nsock_event evt, void *req_v) {
 // Takes a DNS request structure and actually puts it on the wire
 // (calls nsock_write()). Does various other tasks like recording
 // the time for the timeout.
-void put_dns_packet_on_wire(request *req) {
+static void put_dns_packet_on_wire(request *req) {
   char packet[512];
   int plen=0;
   u32 ip;
   struct timeval now, timeout;
 
   ip = (u32) ntohl(req->targ->v4host().s_addr);
-
   packet[0] = (req->id >> 8) & 0xFF;
   packet[1] = req->id & 0xFF;
   plen += 2;
@@ -526,13 +537,18 @@ static int process_result(u32 ia, char *result, int action, u16 id) {
 
       if (id == tpreq->id) {
 
-        if (ia != 0 && (u32) (tpreq->targ->v4host().s_addr) != ia) continue;
+        if (ia != 0 && tpreq->targ->v4host().s_addr != ia)
+          continue;
 
         if (action == ACTION_CNAME_LIST || action == ACTION_FINISHED) {
         tpserv->capacity += CAPACITY_UP_STEP;
         check_capacities(tpserv);
 
-        if (result) tpreq->targ->setHostName(result);
+        if (result) {
+          tpreq->targ->setHostName(result);
+          addto_etchosts(tpreq->targ->v4hostip()->s_addr, result);
+        }
+
         tpserv->in_process.remove(tpreq);
         tpserv->reqs_on_wire--;
 
@@ -568,9 +584,9 @@ static u32 parse_inaddr_arpa(unsigned char *buf, int maxlen) {
   u32 ip=0;
   int i, j;
 
-  for (i=0; i<=3; i++) {
-    if (maxlen <= 0) return 0;
+  if (maxlen <= 0) return 0;
 
+  for (i=0; i<=3; i++) {
     if (buf[0] < 1 || buf[0] > 3) return 0;
 
     maxlen -= buf[0] + 1;
@@ -592,7 +608,7 @@ static u32 parse_inaddr_arpa(unsigned char *buf, int maxlen) {
 // Turns a DNS packet encoded name (see the RFC) and turns it into
 // a normal decimal separated hostname.
 // ASSUMES NAME LENGTH/VALIDITY HAS ALREADY BEEN VERIFIED
-int encoded_name_to_normal(unsigned char *buf, char *output, int outputsize){
+static int encoded_name_to_normal(unsigned char *buf, char *output, int outputsize){
   while (buf[0]) {
     if (buf[0] >= outputsize-1) return -1;
     memcpy(output, buf+1, buf[0]);
@@ -656,9 +672,9 @@ static void read_evt_handler(nsock_pool nsp, nsock_event evt, void *nothing) {
 
   if (nse_type(evt) != NSE_TYPE_READ || nse_status(evt) != NSE_STATUS_SUCCESS) {
     if (o.debugging)
-      log_write(LOG_STDOUT, "mass_dns: warning: got a %s:%s in read_evt_handler()\n",
+      log_write(LOG_STDOUT, "mass_dns: warning: got a %s:%s in %s()\n",
         nse_type2str(nse_type(evt)),
-        nse_status2str(nse_status(evt)));
+        nse_status2str(nse_status(evt)), __func__);
     return;
   }
 
@@ -673,9 +689,10 @@ static void read_evt_handler(nsock_pool nsp, nsock_event evt, void *nothing) {
   // 0xFA == 11111010 (we're not concerned with AA or RD bits)
   if ((buf[2] & 0xFA) != 0x80) return;
 
-  // Check that Recursion is available, the zero field is all zeros
-  // and there is no error condition:
-  if (buf[3] != 0x80) {
+  // Check that the zero field is all zeros and there is no error condition.
+  // We don't care if recursion is available or not since we might be querying
+  // an authoritative DNS server.
+  if (buf[3] != 0x80 && buf[3] != 0) {
     if ((buf[3] & 0xF) == 2) errcode = 2;
     else if ((buf[3] & 0xF) == 3) errcode = 3;
     else return;
@@ -815,9 +832,24 @@ static void add_dns_server(char *ipaddrs) {
 
 }
 
+void free_dns_servers() {
+  std::list<dns_server *>::iterator servI;
+  dns_server *tpserv;
+
+  for(servI = servs.begin(); servI != servs.end();servI++){
+    tpserv = *servI;
+    if(tpserv){
+      if(tpserv->hostname)
+        free(tpserv->hostname);
+      delete tpserv;
+    }
+  }
+  servs.clear();
+}
+
 
 // Creates a new nsi for each DNS server
-void connect_dns_servers() {
+static void connect_dns_servers() {
   std::list<dns_server *>::iterator serverI;
   dns_server *s;
 
@@ -845,10 +877,12 @@ void win32_read_registry(char *controlset) {
   char buf[2048], keyname[2048], *p;
   DWORD sz, i;
 
-  snprintf(keybasebuf, sizeof(keybasebuf), "SYSTEM\\%s\\Services\\Tcpip\\Parameters", controlset);
+  Snprintf(keybasebuf, sizeof(keybasebuf), "SYSTEM\\%s\\Services\\Tcpip\\Parameters", controlset);
   if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, keybasebuf,
-                    0, KEY_READ, &hKey) != ERROR_SUCCESS)
-    fatal("Error opening registry to read DNS servers. Try using --system-dns or specify valid servers with --dns-servers");
+                    0, KEY_READ, &hKey) != ERROR_SUCCESS) {
+    if (firstrun) error("mass_dns: warning: Error opening registry to read DNS servers. Try using --system-dns or specify valid servers with --dns-servers");
+    return;
+  }
 
   sz = sizeof(buf);
   if (RegQueryValueEx(hKey, "NameServer", NULL, NULL, (LPBYTE) buf, (LPDWORD) &sz) == ERROR_SUCCESS)
@@ -860,14 +894,14 @@ void win32_read_registry(char *controlset) {
 
   RegCloseKey(hKey);
 
-  snprintf(keybasebuf, sizeof(keybasebuf), "SYSTEM\\%s\\Services\\Tcpip\\Parameters\\Interfaces", controlset);
+  Snprintf(keybasebuf, sizeof(keybasebuf), "SYSTEM\\%s\\Services\\Tcpip\\Parameters\\Interfaces", controlset);
   if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, keybasebuf,
                     0, KEY_ENUMERATE_SUB_KEYS, &hKey) == ERROR_SUCCESS) {
 
     sz = sizeof(buf);
     for (i=0; RegEnumKeyEx(hKey, i, buf, &sz, NULL, NULL, NULL, NULL) != ERROR_NO_MORE_ITEMS; i++) {
 
-      snprintf(keyname, sizeof(keyname), "SYSTEM\\%s\\Services\\Tcpip\\Parameters\\Interfaces\\%s", controlset, buf);
+      Snprintf(keyname, sizeof(keyname), "SYSTEM\\%s\\Services\\Tcpip\\Parameters\\Interfaces\\%s", controlset, buf);
 
       if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, keyname,
                         0, KEY_READ, &hKey2) == ERROR_SUCCESS) {
@@ -907,7 +941,8 @@ static void parse_resolvdotconf() {
 
   fp = fopen("/etc/resolv.conf", "r");
   if (fp == NULL) {
-    fatal("Unable to open /etc/resolv.conf. Try using --system-dns or specify valid servers with --dns-servers");
+    if (firstrun) error("mass_dns: warning: Unable to open /etc/resolv.conf. Try using --system-dns or specify valid servers with --dns-servers");
+    return;
   }
 
   while (fgets(buf, sizeof(buf), fp)) {
@@ -932,11 +967,10 @@ static void parse_resolvdotconf() {
 }
 
 
-static void parse_etchosts(char *fname) {
+static void parse_etchosts(const char *fname) {
   FILE *fp;
   char buf[2048], hname[256], ipaddrstr[16], *tp;
   struct in_addr ia;
-  host_elem *he;
 
   fp = fopen(fname, "r");
   if (fp == NULL) return; // silently is OK
@@ -953,31 +987,94 @@ static void parse_etchosts(char *fname) {
     while (*tp == ' ' || *tp == '\t') tp++;
 
     if (sscanf(tp, "%15s %255s", ipaddrstr, hname) == 2) {
-      if (inet_aton(ipaddrstr, &ia)) {
-        he = new host_elem;
-        he->name = strdup(hname);
-        he->addr = (u32) ia.s_addr;
-        etchosts[he->addr % HASH_TABLE_SIZE].push_front(he);
-      }
+      if (inet_pton(AF_INET, ipaddrstr, &ia))
+        addto_etchosts(ia.s_addr, hname);
     }
   }
 
   fclose(fp);
 }
 
+void free_etchosts() {
+  host_elem *he;
+  std::list<host_elem *>::iterator hi;
+  int i;
 
+  for(i=0; i < HASH_TABLE_SIZE; i++){
+    for(hi = etchosts[i].begin(); hi != etchosts[i].end(); hi++) {
+      he = *hi;
+      if(he) {
+        free(he->name);
+        delete he;
+      }
+    }
+    etchosts[i].clear();
+  }
+}
+
+/* Executed when the DNS cache is full, ages entries
+ * and removes any with a cache hit of 0 (the least used) */
+bool remove_and_age(host_elem *host) {
+  if(host->cache_hits) {
+     host->cache_hits /=2;
+     return false;
+  } else
+     return true;
+}
+
+/* Add to the dns cache. If there are too many entries
+ * we age and remove the least frequently used ones to
+ * make more space. */
+static void addto_etchosts(u32 ip, const char *hname) {
+  static u16 total_size = 0;
+  std::list<host_elem*>::iterator it;
+  host_elem *he;
+  int i;
+
+  if(lookup_etchosts(ip) != NULL) 
+    return;
+
+  while(total_size >= HASH_TABLE_SIZE) {
+    for(i = 0; i < HASH_TABLE_SIZE; i++) {
+      while((it = find_if(etchosts[i].begin(), etchosts[i].end(), remove_and_age)) != etchosts[i].end()) {
+        etchosts[i].erase(it);
+        /* We don't want total_size to become out of sync with the actual number
+           of entries. */
+        assert(total_size > 0);
+        total_size--;
+      }
+    }
+  }
+  he = new host_elem;
+  he->name = strdup(hname);
+  he->addr = ip;
+  he->cache_hits = 0;
+  etchosts[IP_HASH(ip)].push_back(he);
+  total_size++;
+}
+
+/* Search for a hostname in the cache and increment
+ * its cache hit counter if found */
 static char *lookup_etchosts(u32 ip) {
   std::list<host_elem *>::iterator hostI;
   host_elem *tpelem;
-
-  for(hostI = etchosts[ip % HASH_TABLE_SIZE].begin(); hostI != etchosts[ip % HASH_TABLE_SIZE].end(); hostI++) {
+  int localIP_Hash = IP_HASH(ip);
+  for(hostI = etchosts[localIP_Hash].begin(); hostI != etchosts[localIP_Hash].end(); hostI++) {
     tpelem = *hostI;
-    if (tpelem->addr == ip) return tpelem->name;
+    if (tpelem->addr == ip) {
+      if(tpelem->cache_hits < UCHAR_MAX)
+        tpelem->cache_hits++;
+      return tpelem->name;
+    }
   }
-
   return NULL;
 }
 
+/* External interface to dns cache */
+const char *lookup_cached_host(u32 ip) {
+  const char *tmp = lookup_etchosts(ip);
+  return tmp;
+}
 
 static void etchosts_init(void) {
   static int initialized = 0;
@@ -996,11 +1093,11 @@ static void etchosts_init(void) {
   has_backslash = (windows_dir[strlen(windows_dir)-1] == '\\');
 
   // Windows 95/98/Me:
-  snprintf(tpbuf, sizeof(tpbuf), "%s%shosts", windows_dir, has_backslash ? "" : "\\");
+  Snprintf(tpbuf, sizeof(tpbuf), "%s%shosts", windows_dir, has_backslash ? "" : "\\");
   parse_etchosts(tpbuf);
 
   // Windows NT/2000/XP/2K3:
-  snprintf(tpbuf, sizeof(tpbuf), "%s%ssystem32\\drivers\\etc\\hosts", windows_dir, has_backslash ? "" : "\\");
+  Snprintf(tpbuf, sizeof(tpbuf), "%s%ssystem32\\drivers\\etc\\hosts", windows_dir, has_backslash ? "" : "\\");
   parse_etchosts(tpbuf);
 
 #else
@@ -1022,6 +1119,7 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
   char *tpname;
   int i;
   bool lasttrace = false;
+  char spmobuf[1024];
 
   if (o.mass_dns == false) {
     Target *currenths;
@@ -1035,8 +1133,9 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
       if (((currenths->flags & HOST_UP) || o.resolve_all) && !o.noresolve) stat_actual++;
     }
 
-    SPM = new ScanProgressMeter("System DNS resolution");
-    
+    Snprintf(spmobuf, sizeof(spmobuf), "System DNS resolution of %d host%s.", num_targets, num_targets-1 ? "s" : "");
+    SPM = new ScanProgressMeter(spmobuf);
+
     for(i=0, hostI = targets; hostI < targets+num_targets; hostI++, i++) {
       currenths = *hostI;
 	
@@ -1054,6 +1153,7 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
       }
     }
 
+    SPM->endTask(NULL, NULL);
     delete SPM;
 
     return;
@@ -1064,8 +1164,7 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
     if (o.dns_servers) add_dns_server(o.dns_servers);
     else parse_resolvdotconf();
 
-    if (servs.size() == 0)
-      fatal("Unable to determine any DNS servers. Try using --system-dns or specify valid servers with --dns_servers");
+    if (servs.size() == 0 && firstrun) error("mass_dns: warning: Unable to determine any DNS servers. Reverse DNS is disabled. Try using --system-dns or specify valid servers with --dns_servers");
   }
 
 
@@ -1080,7 +1179,7 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
   for(hostI = targets; hostI < targets+num_targets; hostI++) {
     if (!((*hostI)->flags & HOST_UP) && !o.resolve_all) continue;
 
-    // See if it's in /etc/hosts
+    // See if it's in /etc/hosts or cached
     tpname = lookup_etchosts((u32) (*hostI)->v4hostip()->s_addr);
     if (tpname) {
       (*hostI)->setHostName(tpname);
@@ -1099,12 +1198,12 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
     total_reqs++;
   }
 
-  if (total_reqs == 0) return;
+  if (total_reqs == 0 || servs.size() == 0) return;
 
   // And finally, do it!
 
   if ((dnspool = nsp_new(NULL)) == NULL)
-    fatal("Unable to create nsock pool in nmap_mass_rdns_core()");
+    fatal("Unable to create nsock pool in %s()", __func__);
 
   if ((lasttrace = o.packetTrace()))
     nsp_settrace(dnspool, 5, o.getStartTime());
@@ -1115,7 +1214,8 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
 
   read_timeout_index = MIN(sizeof(read_timeouts)/sizeof(read_timeouts[0]), servs.size()) - 1;
 
-  SPM = new ScanProgressMeter("System DNS resolution");
+  Snprintf(spmobuf, sizeof(spmobuf), "Parallel DNS resolution of %d host%s.", num_targets, num_targets-1 ? "s" : "");
+  SPM = new ScanProgressMeter(spmobuf);
 
   while (total_reqs > 0) {
     timeout = deal_with_timedout_reads();
@@ -1134,6 +1234,7 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
     nsock_loop(dnspool, timeout);
   }
 
+  SPM->endTask(NULL, NULL);
   delete SPM;
 
   close_dns_servers();
@@ -1143,34 +1244,38 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
   if (cname_reqs.size() && o.debugging)
     log_write(LOG_STDOUT, "Performing system-dns for %d domain names that use CNAMEs\n", (int) cname_reqs.size());
 
-  SPM = new ScanProgressMeter("System CNAME DNS resolution");
+  if (cname_reqs.size()) {
+    Snprintf(spmobuf, sizeof(spmobuf), "System CNAME DNS resolution of %u host%s.", (unsigned) cname_reqs.size(), cname_reqs.size()-1 ? "s" : "");
+    SPM = new ScanProgressMeter(spmobuf);
 
-  for(i=0, reqI = cname_reqs.begin(); reqI != cname_reqs.end(); reqI++, i++) {
-    struct sockaddr_storage ss;
-    size_t sslen;
-    char hostname[MAXHOSTNAMELEN + 1] = "";
+    for(i=0, reqI = cname_reqs.begin(); reqI != cname_reqs.end(); reqI++, i++) {
+      struct sockaddr_storage ss;
+      size_t sslen;
+      char hostname[MAXHOSTNAMELEN + 1] = "";
 
-    if (keyWasPressed())
-      SPM->printStats((double) i / cname_reqs.size(), NULL);
+      if (keyWasPressed())
+        SPM->printStats((double) i / cname_reqs.size(), NULL);
 
-    tpreq = *reqI;
+      tpreq = *reqI;
 
-    if (tpreq->targ->TargetSockAddr(&ss, &sslen) != 0)
-      fatal("Failed to get target socket address.");
+      if (tpreq->targ->TargetSockAddr(&ss, &sslen) != 0)
+        fatal("Failed to get target socket address.");
+
+      if (getnameinfo((struct sockaddr *)&ss, sslen, hostname,
+                      sizeof(hostname), NULL, 0, NI_NAMEREQD) == 0) {
+        stat_ok++;
+        stat_cname++;
+        tpreq->targ->setHostName(hostname);
+      }
+
+      delete tpreq;
 
-    if (getnameinfo((struct sockaddr *)&ss, sslen, hostname,
-                    sizeof(hostname), NULL, 0, NI_NAMEREQD) == 0) {
-      stat_ok++;
-      stat_cname++;
-      tpreq->targ->setHostName(hostname);
     }
 
-    delete tpreq;
-
+    SPM->endTask(NULL, NULL);
+    delete SPM;
   }
 
-  delete SPM;
-
   cname_reqs.clear();
 
 }
@@ -1191,22 +1296,25 @@ void nmap_mass_rdns(Target **targets, int num_targets) {
 
   gettimeofday(&now, NULL);
 
-  if (o.verbose || o.debugging) {
-    if (o.mass_dns) {
-      // #:  Number of DNS servers used
-      // OK: Number of fully reverse resolved queries
-      // NX: Number of confirmations of 'No such reverse domain eXists'
-      // DR: Dropped IPs (no valid responses were received)
-      // SF: Number of IPs that got 'Server Failure's
-      // TR: Total number of transmissions necessary. The number of domains is ideal, higher is worse
-      log_write(LOG_STDOUT, "DNS resolution of %d IPs took %.2fs. Mode: Async [#: %lu, OK: %d, NX: %d, DR: %d, SF: %d, TR: %d, CN: %d]\n",
-                    stat_actual, TIMEVAL_MSEC_SUBTRACT(now, starttv) / 1000.0,
-                    (unsigned long) servs.size(), stat_ok, stat_nx, stat_dropped, stat_sf, stat_trans, stat_cname);
-    } else {
-      log_write(LOG_STDOUT, "DNS resolution of %d IPs took %.2fs. Mode: System [OK: %d, ??: %d]\n",
-                    stat_actual, TIMEVAL_MSEC_SUBTRACT(now, starttv) / 1000.0,
-                    stat_ok, stat_actual - stat_ok);
+  if (stat_actual > 0) {
+    if (o.debugging || o.verbose >= 3) {
+      if (o.mass_dns) {
+	// #:  Number of DNS servers used
+	// OK: Number of fully reverse resolved queries
+	// NX: Number of confirmations of 'No such reverse domain eXists'
+	// DR: Dropped IPs (no valid responses were received)
+	// SF: Number of IPs that got 'Server Failure's
+	// TR: Total number of transmissions necessary. The number of domains is ideal, higher is worse
+	log_write(LOG_STDOUT, "DNS resolution of %d IPs took %.2fs. Mode: Async [#: %lu, OK: %d, NX: %d, DR: %d, SF: %d, TR: %d, CN: %d]\n",
+		  stat_actual, TIMEVAL_MSEC_SUBTRACT(now, starttv) / 1000.0,
+		  (unsigned long) servs.size(), stat_ok, stat_nx, stat_dropped, stat_sf, stat_trans, stat_cname);
+      } else {
+	log_write(LOG_STDOUT, "DNS resolution of %d IPs took %.2fs. Mode: System [OK: %d, ??: %d]\n",
+		  stat_actual, TIMEVAL_MSEC_SUBTRACT(now, starttv) / 1000.0,
+		  stat_ok, stat_actual - stat_ok);
+      }
     }
   }
 
+  firstrun=0;
 }
