@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2004 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2008 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -14,6 +14,15 @@
  * may be willing to sell alternative licenses (contact                    *
  * sales@insecure.com ).                                                   *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         * 
  * If you received these files with a written license agreement stating    *
  * terms other than the (GPL) terms above, then that alternative license   *
  * agreement takes precedence over this comment.                          *
@@ -26,7 +35,7 @@
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
+ * distribution.  By sending these changes to Fyodor or one of the         *
  * insecure.org development mailing lists, it is assumed that you are      *
  * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
  * to reuse, modify, and relicense the code.  Nmap will always be          *
@@ -40,32 +49,41 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details (                               *
- * http://www.gnu.org/copyleft/gpl.html ).                                 *
+ * General Public License v2.0 for more details                            *
+ * (http://www.gnu.org/licenses/gpl-2.0.html).                             *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock.h,v 1.21 2004/03/12 01:50:58 fyodor Exp $ */
+/* $Id: nsock.h 7631 2008-05-22 18:14:15Z kris $ */
+
+/* Would you like to include pcap support in nsock?
+ * Pcap support code is currently unstable, so we give
+ * you a choice. In future this #define will be removed.*/
+#define HAVE_PCAP 1
 
 #ifndef NSOCK_H
 #define NSOCK_H
 
+/* Keep assert() defined for security reasons */
+#undef NDEBUG
+
 #include <sys/types.h>
 #ifndef WIN32
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <time.h>
+#include <sys/time.h>
 #endif
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-// The read calls will generally return after reading at least this
-// much data so that the caller can process it and so that the
-// connection spewing data doesn't monopolize resources.  The caller
-// can always initiate another read request to ask for more.
+/* The read calls will generally return after reading at least this
+ * much data so that the caller can process it and so that the
+ * connection spewing data doesn't monopolize resources.  The caller
+ * can always initiate another read request to ask for more.
+ */
 #define NSOCK_READ_CHUNK_SIZE 0x8FFFF
 
 /*********************  TYPEDEFS ********************/
@@ -124,11 +142,12 @@ int nsp_geterrorcode(nsock_pool nsp);
 /* Every nsp has an ID that is unique across the program execution */
 unsigned long nsp_getid(nsock_pool nsp);
 
-// Note that nsi_get1_ssl_session will increment the usage count
-// of the SSL_SESSION, since nsock does a free when the nsi is
-// destroyed.  It's up to any calling function/etc to do a
-// SSL_SESSION_free() on it.  nsi_get0_ssl_session doesn't
-// increment, and is for informational purposes only.
+/* Note that nsi_get1_ssl_session will increment the usage count
+ * of the SSL_SESSION, since nsock does a free when the nsi is
+ * destroyed.  It's up to any calling function/etc to do a
+ * SSL_SESSION_free() on it.  nsi_get0_ssl_session doesn't
+ * increment, and is for informational purposes only.
+ */
 nsock_ssl_session nsi_get1_ssl_session(nsock_iod nsockiod);
 nsock_ssl_session nsi_get0_ssl_session(nsock_iod nsockiod);
 
@@ -168,9 +187,15 @@ void nsp_delete(nsock_pool nsp);
 /* IF YOU ADD NEW NSE_TYPES YOU MUST INCREASE TYPE_CODE_NUM_BITS SO THAT
    IT IS ALWAYS log2(maximum_nse_type_value + 1) */
 #define TYPE_CODE_NUM_BITS 3
-enum nse_type { NSE_TYPE_CONNECT=0, NSE_TYPE_CONNECT_SSL=1, NSE_TYPE_READ=2, NSE_TYPE_WRITE=3, 
-		NSE_TYPE_TIMER=4 };  /* At some point I was considering a 
-				      NSE_TYPE_START and NSE_TYPE_CUSTOM */
+enum nse_type { 
+	NSE_TYPE_CONNECT=0,
+	NSE_TYPE_CONNECT_SSL=1,
+	NSE_TYPE_READ=2,
+	NSE_TYPE_WRITE=3, 
+	NSE_TYPE_TIMER=4,
+	NSE_TYPE_PCAP_READ=5,
+	NSE_TYPE_MAX=6,
+};  /* At some point I was considering a NSE_TYPE_START and NSE_TYPE_CUSTOM */
 
 /* Find the type of an event that spawned a callback */
 enum nse_type nse_type(nsock_event nse);
@@ -391,6 +416,14 @@ nsock_event_id nsock_connect_ssl(nsock_pool nsp, nsock_iod nsiod,
 				 size_t sslen, unsigned short port,
 				 nsock_ssl_session ssl_session);
 
+/* Request ssl connection over already established TCP connection.
+   nsiod must be socket that is already connected to target
+   using nsock_connect_tcp.
+   All parameters have the same meaning as in 'nsock_connect_ssl' */
+nsock_event_id nsock_reconnect_ssl(nsock_pool nsp, nsock_iod nsiod,
+				   nsock_ev_handler handler, int timeout_msecs,
+                                   void *userdata, nsock_ssl_session ssl_session);
+
 /* Read up to nlines lines (terminated with \n, which of course
    inclues \r\n), or until EOF, or until the timeout, whichever comes
    first.  Note that NSE_STATUS_SUCCESS will be returned in the case
@@ -443,6 +476,54 @@ int nsock_event_cancel(nsock_pool ms_pool, nsock_event_id id, int notify );
    before returning */
 const struct timeval *nsock_gettimeofday();
 
+
+#ifdef HAVE_PCAP
+/*
+ * Open pcap device and connect it to nsp. Other parameters have the
+ * same meaning as for pcap_open_live in pcap(3).
+ * 	device   : pcap-style device name
+ * 	snaplen  : size of packet to be copied to handler
+ * 	promisc  : whether to open device in promiscous mode
+ * 	bpf_fmt	 : berkeley filter 
+ * return value: NULL if everything was okay, or error string if error occurred
+ * [sorry Fyodor for breaking the API, but it's just simpler]
+ * */
+char *nsock_pcap_open(nsock_pool nsp, nsock_iod nsiod, 
+		const char *pcap_device, int snaplen, int promisc,
+		const char *bpf_fmt, ...);
+
+/*
+ * Requests exacly one packet to be captured.from pcap. 
+ * See nsock_read() for parameters description.
+ * */
+nsock_event_id nsock_pcap_read_packet(nsock_pool nsp, nsock_iod nsiod, 
+			 nsock_ev_handler handler, int timeout_msecs,
+			 void *userdata);
+
+/*
+ * Gets packet data. This should be called after succesfull receiving of packet
+ * to get packet.  If you're not interested in some values, just pass NULL
+ * instead of valid pointer.
+ * l3_data is just after l2_data in buffer. Feel free to treat l2_data as one
+ * buffer with size of (l2_len + l3_len). 
+ * Ts time is fixed for systems that don't support proper timing, like Windows.
+ * So TS is pointing to time when packet was received or to the time _after_.
+ * As a result you'll get longer times than you should, but it's safer to
+ * think that host is a bit further. 
+ * */
+void nse_readpcap(nsock_event nsee,
+	const unsigned char **l2_data, size_t *l2_len,
+	const unsigned char **l3_data, size_t *l3_len,
+	size_t *packet_len, struct timeval *ts); 
+
+/* Well. Just pcap-style datalink. Like DLT_EN10MB or DLT_SLIP. Check in pcap(3) manpage. */
+int nsi_pcap_linktype(nsock_iod nsiod);
+
+/* Is this nsiod a pcap descriptor? */
+int nsi_is_pcap(nsock_iod nsiod);
+
+#endif /* HAVE_PCAP */
+
 #ifdef __cplusplus
 } /* End of 'extern "C"' */
 #endif
@@ -455,3 +536,4 @@ const struct timeval *nsock_gettimeofday();
 
 
 
+
