@@ -4,7 +4,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2004 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2008 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -16,7 +16,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -34,7 +34,7 @@
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
+ * distribution.  By sending these changes to Fyodor or one of the         *
  * insecure.org development mailing lists, it is assumed that you are      *
  * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
  * to reuse, modify, and relicense the code.  Nmap will always be          *
@@ -48,12 +48,12 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details (                               *
- * http://www.gnu.org/copyleft/gpl.html ).                                 *
+ * General Public License v2.0 for more details                            *
+ * (http://www.gnu.org/licenses/gpl-2.0.html).                             *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock_core.c 2535 2005-01-31 20:40:45Z fyodor $ */
+/* $Id: nsock_core.c 7592 2008-05-21 06:20:33Z mixter $ */
 
 #include "nsock_internal.h"
 #include "gh_list.h"
@@ -81,6 +81,11 @@
 
 #include "netutils.h"
 
+#if HAVE_PCAP
+#include "nsock_pcap.h"
+static int pcap_read_on_nonselect(mspool *nsp);
+#endif
+
 /* Msock time of day -- we update this at least once per
    nsock_loop round (and after most calls that are likely to block).  
    Other nsock files should grab this 
@@ -104,18 +109,21 @@ static int wait_for_events(mspool *ms, int msec_timeout) {
   if (ms->evl.events_pending == 0)
     return 0; /* No need to wait on 0 events ... */
 
-  do {  
+  do {
+    if (ms->tracelevel > 3)
+        nsock_trace(ms, "wait_for_events");
+        
     if (ms->evl.next_ev.tv_sec == 0) {
       event_msecs = -1; /* None of the events specified a timeout */
     } else {
-      event_msecs = MAX(0, TIMEVAL_NSEC_SUBTRACT(ms->evl.next_ev, nsock_tod));
+      event_msecs = MAX(0, TIMEVAL_MSEC_SUBTRACT(ms->evl.next_ev, nsock_tod));
     }
 
     /* We cast to unsigned because we want -1 to be very high (since it means 
        no timeout) */
     combined_msecs = MIN((unsigned) event_msecs, (unsigned) msec_timeout);
     
-    // printf("wait_for_events: starting wait -- combined_msecs=%d\n", combined_msecs);
+    /* printf("wait_for_events: starting wait -- combined_msecs=%d\n", combined_msecs); */
     /* Set up the timeval pointer we will give to select() */
     memset(&select_tv, 0, sizeof(select_tv));
     if (combined_msecs > 0) {
@@ -131,6 +139,7 @@ static int wait_for_events(mspool *ms, int msec_timeout) {
       select_tv_p = NULL;
     }
     
+    
     /* Figure out whether there are any FDs in the sets, as @$@!$#
        Windows returns WSAINVAL (10022) if you call a select() with no
        FDs, even though the Linux man page says that doing so is a
@@ -144,24 +153,31 @@ static int wait_for_events(mspool *ms, int msec_timeout) {
       else ms->mioi.max_sd--;
     }
 
-    if (ms->mioi.max_sd < 0) {
-      ms->mioi.results_left = 0;
-      if (combined_msecs > 0)
-	usleep(combined_msecs * 1000);
-    } else {
+#if HAVE_PCAP
+    /* do non-blocking read on pcap devices that doesn't support select() 
+     * If there is anything read, don't do usleep() or select(), just leave this loop */
+    if (pcap_read_on_nonselect(ms)) {
+    	/* okay, something was read. */
+    } else 
+#endif
+      if (ms->mioi.max_sd < 0) {
+        ms->mioi.results_left = 0;
+        if (combined_msecs > 0)
+	  usleep(combined_msecs * 1000);
+      } else {
 
-      /* Set up the descriptors for select */
-      ms->mioi.fds_results_r = ms->mioi.fds_master_r;
-      ms->mioi.fds_results_w = ms->mioi.fds_master_w;
-      ms->mioi.fds_results_x = ms->mioi.fds_master_x;
+        /* Set up the descriptors for select */
+        ms->mioi.fds_results_r = ms->mioi.fds_master_r;
+        ms->mioi.fds_results_w = ms->mioi.fds_master_w;
+        ms->mioi.fds_results_x = ms->mioi.fds_master_x;
       
-      ms->mioi.results_left = select(ms->mioi.max_sd + 1, &ms->mioi.fds_results_r, &ms->mioi.fds_results_w, &ms->mioi.fds_results_x, select_tv_p);
-      if (ms->mioi.results_left == -1)
-	sock_err = socket_errno();
-    }
+        ms->mioi.results_left = select(ms->mioi.max_sd + 1, &ms->mioi.fds_results_r, &ms->mioi.fds_results_w, &ms->mioi.fds_results_x, select_tv_p);
+        if (ms->mioi.results_left == -1)
+  	  sock_err = socket_errno();
+      }
 
     gettimeofday(&nsock_tod, NULL); /* Due to usleep or select delay */
-  } while (ms->mioi.results_left == -1 && sock_err == EINTR);
+  } while (ms->mioi.results_left == -1 && sock_err == EINTR);// repeat only if signal occured
   
   if (ms->mioi.results_left == -1 && sock_err != EINTR) {
     ms->errnum = sock_err;
@@ -209,7 +225,8 @@ static int wait_for_events(mspool *ms, int msec_timeout) {
 void handle_connect_result(mspool *ms, msevent *nse, 
 				  enum nse_status status)
   {
-  int optval, optlen = sizeof(int);
+  int optval;
+  socklen_t optlen = sizeof(int);
   char buf[1024];
   msiod *iod = nse->iod;
 #if HAVE_OPENSSL
@@ -268,7 +285,7 @@ void handle_connect_result(mspool *ms, msevent *nse,
       nse->errnum = optval;
       break;
     default:
-      snprintf(buf, sizeof(buf), "Strange connect error from %s (%d)", inet_ntop_ez(&iod->peer, iod->peerlen), optval);
+      Snprintf(buf, sizeof(buf), "Strange connect error from %s (%d)", inet_ntop_ez(&iod->peer, iod->peerlen), optval);
       perror(buf);
       assert(0); /* I'd like for someone to report it */
       break;
@@ -458,6 +475,9 @@ static int do_actual_read(mspool *ms, msevent *nse) {
     /* Traditional read() - no SSL - using recv() because that works better on Windows */
     do {
       buflen = recv(iod->sd, buf, sizeof(buf), 0);
+      /* Using recv() was failing, at least on UNIX, for non-network sockets (i.e. stdin)
+       * in this case, a read() is done - as on ENOTSOCK we may have a non-network socket */
+      if (buflen == -1 && socket_errno() == ENOTSOCK) buflen = read(iod->sd, buf, sizeof(buf));
       if (buflen == -1) err = socket_errno();
       if (buflen > 0) {
 	if (fscat(&nse->iobuf, buf, buflen) == -1) {
@@ -467,15 +487,17 @@ static int do_actual_read(mspool *ms, msevent *nse) {
 	  return -1;
 	}
 
-	// Sometimes a service just spews and spews data.  So we return
-	// after a somewhat large amount to avoid monopolizing resources
-	// and avoid DOS attacks.
+	/* Sometimes a service just spews and spews data.  So we return
+	 * after a somewhat large amount to avoid monopolizing resources
+	 * and avoid DOS attacks.
+	 */
 	if (FILESPACE_LENGTH(&nse->iobuf) > max_chunk)
 	  return FILESPACE_LENGTH(&nse->iobuf) - startlen;
 	
-	// No good reason to read again if we we were successful in the read but
-	// didn't fill up the buffer.  I'll insist on it being TCP too, because I
-	// think UDP might only give me one packet worth at a time (I dunno ...).
+	/* No good reason to read again if we we were successful in the read but
+	 * didn't fill up the buffer.  I'll insist on it being TCP too, because I
+	 * think UDP might only give me one packet worth at a time (I dunno ...).
+	 */
 	if (buflen > 0 && buflen < sizeof(buf) && iod->lastproto == IPPROTO_TCP)
 	  return FILESPACE_LENGTH(&nse->iobuf) - startlen;
       }
@@ -501,9 +523,10 @@ static int do_actual_read(mspool *ms, msevent *nse) {
 	return -1;
       }
       
-      // Sometimes a service just spews and spews data.  So we return
-      // after a somewhat large amount to avoid monopolizing resources
-      // and avoid DOS attacks.
+      /* Sometimes a service just spews and spews data.  So we return
+       * after a somewhat large amount to avoid monopolizing resources
+       * and avoid DOS attacks.
+       */
       if (FILESPACE_LENGTH(&nse->iobuf) > NSOCK_READ_CHUNK_SIZE)
 	return FILESPACE_LENGTH(&nse->iobuf) - startlen;
     }
@@ -546,9 +569,9 @@ void handle_read_result(mspool *ms, msevent *nse,
 			       enum nse_status status)  
 {
 
-  unsigned int count, len;
+  unsigned int count;
   char *str;
-  int rc;
+  int rc, len;
   msiod *iod = nse->iod;
 
   if (status == NSE_STATUS_TIMEOUT) {
@@ -621,8 +644,9 @@ void handle_read_result(mspool *ms, msevent *nse,
       FD_CLR(iod->sd, &ms->mioi.fds_results_w);
     }
 
-    // Note -- I only want to decrement IOD if there are no other events hinging on it. 
-    // For example, there could be a READ and WRITE outstanding at once
+    /* Note -- I only want to decrement IOD if there are no other events hinging on it. 
+     * For example, there could be a READ and WRITE outstanding at once
+     */
     if (iod->events_pending <= 1 && ms->mioi.max_sd == iod->sd)
       ms->mioi.max_sd--;
   }
@@ -645,6 +669,73 @@ void handle_read_result(mspool *ms, msevent *nse,
   return;
 }
 
+#if HAVE_PCAP
+void handle_pcap_read_result(mspool *ms, msevent *nse, 
+			       enum nse_status status)  
+{
+  msiod *iod = nse->iod;
+  mspcap *mp = (mspcap *) iod->pcap;
+  
+
+  if (status == NSE_STATUS_TIMEOUT) {
+    nse->status = NSE_STATUS_TIMEOUT;
+    nse->event_done = 1;
+  } else if (status == NSE_STATUS_CANCELLED) {
+    nse->status = NSE_STATUS_CANCELLED;
+    nse->event_done = 1;
+  } else if (status == NSE_STATUS_SUCCESS) {
+    /* check if we already have something read */
+    if(FILESPACE_LENGTH(&(nse->iobuf)) == 0){
+	nse->status = NSE_STATUS_TIMEOUT;
+        nse->event_done = 0;
+    }else{
+    	nse->status = NSE_STATUS_SUCCESS;/* we have full buffer */
+    	nse->event_done = 1;
+    }
+  } else {
+    assert(0); /* Currently we only know about TIMEOUT, CANCELLED, and SUCCESS callbacks */
+  }
+  
+  /* If we asked for an event dispatch, we are done reading on the socket so 
+     we can take it off the descriptor list ... */
+  if (nse->event_done && mp->pcap_desc >= 0) {
+      FD_CLR(mp->pcap_desc, &ms->mioi.fds_master_r);
+      FD_CLR(mp->pcap_desc, &ms->mioi.fds_results_r);
+  }
+  return;
+}
+
+/*
+ * returns number of descriptors on which data was read
+ * */
+static int pcap_read_on_nonselect(mspool *nsp)
+{
+	gh_list *event_list = &nsp->evl.pcap_read_events;
+	gh_list_elem *current, *next;
+	msevent *nse;
+	int rc;
+	int ret = 0;
+	
+	
+	for(current = GH_LIST_FIRST_ELEM(event_list);
+		current != NULL; current = next) {
+		
+		nse = (msevent *) GH_LIST_ELEM_DATA(current);
+		rc = do_actual_pcap_read(nse);
+		if(rc==1){ /* something received */
+		  ret++;
+		  break;
+	        }
+		next = GH_LIST_ELEM_NEXT(current);
+	}
+
+	return(ret);
+}
+#endif // HAVE_PCAP
+
+
+
+
   /* Iterate through all the event lists (such as 
      connect_events, read_events, timer_events, etc) and take action
      for those that have completed (due to timeout, i/o, etc) */
@@ -660,6 +751,9 @@ static void iterate_through_event_lists(mspool *nsp) {
                                &nsp->evl.read_events,
                                &nsp->evl.write_events,
                                &nsp->evl.timer_events,
+			       #if HAVE_PCAP
+			       &nsp->evl.pcap_read_events,
+			       #endif                               
                                0
                              };
     int current_list_idx;
@@ -676,15 +770,28 @@ static void iterate_through_event_lists(mspool *nsp) {
 	leads to write().
   */
 
+    /* foreach list */
+    if (nsp->tracelevel > 7){
+      for(current_list_idx = 0; event_lists[current_list_idx] != NULL;
+	current_list_idx++) {
+        nsock_trace(nsp, "before iterating, list %i", current_list_idx);
+        for(current = GH_LIST_FIRST_ELEM(event_lists[current_list_idx]);
+	    current != NULL; current = GH_LIST_ELEM_NEXT(current)) {
+	  nse = (msevent *) GH_LIST_ELEM_DATA(current);
+	  nsock_trace(nsp, "before iterating %lu",nse->id);
+        }
+      }
+    }
     /* foreach list */
     for(current_list_idx = 0; event_lists[current_list_idx] != NULL;
 	current_list_idx++) {
-
     /* foreach element in the list */
       for(current = GH_LIST_FIRST_ELEM(event_lists[current_list_idx]);
 	  current != NULL; current = next) {
-
 	nse = (msevent *) GH_LIST_ELEM_DATA(current);
+        if (nsp->tracelevel > 7)
+	  nsock_trace(nsp, "list %i, iterating %lu",current_list_idx, nse->id);
+
 	if ( ! nse->event_done) {	
 	  switch(nse->type) {
 	  case NSE_TYPE_CONNECT:
@@ -693,9 +800,9 @@ static void iterate_through_event_lists(mspool *nsp) {
 		FD_ISSET(nse->iod->sd, &nsp->mioi.fds_results_w) ||
 		FD_ISSET(nse->iod->sd, &nsp->mioi.fds_results_x)) {
 	      handle_connect_result(nsp, nse, NSE_STATUS_SUCCESS);
-	    } 
+	    }
 	    if (!nse->event_done && nse->timeout.tv_sec &&
-		TIMEVAL_NSEC_SUBTRACT(nse->timeout, nsock_tod) <= 0) {
+		TIMEVAL_MSEC_SUBTRACT(nse->timeout, nsock_tod) <= 0) {
 	      handle_connect_result(nsp, nse, NSE_STATUS_TIMEOUT);
 	    }
 	    break;
@@ -714,7 +821,7 @@ static void iterate_through_event_lists(mspool *nsp) {
 	      handle_read_result(nsp, nse, NSE_STATUS_SUCCESS);
 	    
 	    if (!nse->event_done && nse->timeout.tv_sec &&
-		TIMEVAL_NSEC_SUBTRACT(nse->timeout, nsock_tod) <= 0) {
+		TIMEVAL_MSEC_SUBTRACT(nse->timeout, nsock_tod) <= 0) {
 	      handle_read_result(nsp, nse, NSE_STATUS_TIMEOUT);
 	    }
 	    break;
@@ -734,18 +841,61 @@ static void iterate_through_event_lists(mspool *nsp) {
 	      handle_write_result(nsp, nse, NSE_STATUS_SUCCESS);
 
 	    if (!nse->event_done && nse->timeout.tv_sec &&
-		TIMEVAL_NSEC_SUBTRACT(nse->timeout, nsock_tod) <= 0) {
+		TIMEVAL_MSEC_SUBTRACT(nse->timeout, nsock_tod) <= 0) {
 	      handle_write_result(nsp, nse, NSE_STATUS_TIMEOUT);
 	    }
 	    break;
 	    
 	  case NSE_TYPE_TIMER:
 	    if (nse->timeout.tv_sec && 
-		TIMEVAL_NSEC_SUBTRACT(nse->timeout, nsock_tod) <= 0) {
+		TIMEVAL_MSEC_SUBTRACT(nse->timeout, nsock_tod) <= 0) {
 	      handle_timer_result(nsp, nse, NSE_STATUS_SUCCESS);
 	    }
 	    break;
+
+#if HAVE_PCAP
+	  case NSE_TYPE_PCAP_READ:{
+	    if (nsp->tracelevel > 5)
+		    nsock_trace(nsp, "PCAP iterating %lu",nse->id);
+
+	    /* buffer empty? check it! */
+	    if ( FILESPACE_LENGTH(&(nse->iobuf))==0 ) 
+		do_actual_pcap_read(nse);
+
+	    /* if already received smth */
+	    if ( FILESPACE_LENGTH(&(nse->iobuf))>0 )
+	      handle_pcap_read_result(nsp, nse, NSE_STATUS_SUCCESS);
 	    
+	    if (!nse->event_done && nse->timeout.tv_sec &&
+		TIMEVAL_MSEC_SUBTRACT(nse->timeout, nsock_tod) <= 0)
+	      handle_pcap_read_result(nsp, nse, NSE_STATUS_TIMEOUT);
+	    
+	    #if PCAP_BSD_SELECT_HACK
+	    /* If event occured, and we're in BSD_HACK mode, than this event was added
+	     * to two queues. evl.read_event and evl.pcap_read_event 
+	     * Of coure we should destroy it only once. 
+	     * I assume we're now in evl.read_event, co just unlink this event from
+	     * evl.pcap_read_event */
+	    if(((mspcap *) nse->iod->pcap)->pcap_desc >= 0 &&
+	       nse->event_done &&
+	       event_lists[current_list_idx] == &nsp->evl.read_events){
+	      /* event is done, list is read_events and we're in BSD_HACK mode. 
+	       * So unlink event from pcap_read_events */
+	      gh_list_remove(&nsp->evl.pcap_read_events, nse); 
+              if (nsp->tracelevel > 8)
+                  nsock_trace(nsp, "PCAP NSE #%lu: Removing event from PCAP_READ_EVENTS", nse->id);
+	    }
+	    if(((mspcap *) nse->iod->pcap)->pcap_desc >= 0 &&
+	       nse->event_done &&
+	       event_lists[current_list_idx] == &nsp->evl.pcap_read_events){
+ 	      gh_list_remove(&nsp->evl.read_events, nse); 
+              if (nsp->tracelevel > 8)
+                  nsock_trace(nsp, "PCAP NSE #%lu: Removing event from READ_EVENTS", nse->id);
+	    }
+	    #endif
+	    break;
+	  }
+#endif
 	  default:
 	    fatal("Event has unknown type (%d)", nse->type);
 	    break; /* unreached */
@@ -753,6 +903,9 @@ static void iterate_through_event_lists(mspool *nsp) {
 	}
 
 	if (nse->event_done) {
+          if (nsp->tracelevel > 8)
+            nsock_trace(nsp, "NSE #%lu: Removing event from event_lists[%i]", nse->id, current_list_idx);
+
 	  /* WooHoo!  The event is ready to be sent */
 	  msevent_dispatch_and_delete(nsp, nse, 1);
 	  next = GH_LIST_ELEM_NEXT(current);
@@ -763,7 +916,7 @@ static void iterate_through_event_lists(mspool *nsp) {
 	  if (nse->timeout.tv_sec != 0) {
 	    if (nsp->evl.next_ev.tv_sec == 0)
 	      nsp->evl.next_ev = nse->timeout;
-	    else if (TIMEVAL_NSEC_SUBTRACT(nsp->evl.next_ev, nse->timeout) > 0)
+	    else if (TIMEVAL_MSEC_SUBTRACT(nsp->evl.next_ev, nse->timeout) > 0)
 	      nsp->evl.next_ev = nse->timeout;
 	  }
 	}
@@ -789,12 +942,11 @@ enum nsock_loopstatus quitstatus = NSOCK_LOOP_ERROR;
 
 gettimeofday(&nsock_tod, NULL);
 
- if (msec_timeout > 0) { 
-   TIMEVAL_NSEC_ADD(loop_timeout, nsock_tod, msec_timeout);
- } else if (msec_timeout < -1) {
+ if (msec_timeout < -1) {
    ms->errnum = EINVAL;
    return NSOCK_LOOP_ERROR;
  }
+ TIMEVAL_MSEC_ADD(loop_timeout, nsock_tod, msec_timeout);
  msecs_left = msec_timeout;
  
  if (ms->tracelevel > 1) {
@@ -816,7 +968,7 @@ while(1) {
   }
  
   if (msec_timeout > 0) {
-    msecs_left = MAX(0, TIMEVAL_NSEC_SUBTRACT(loop_timeout, nsock_tod));
+    msecs_left = MAX(0, TIMEVAL_MSEC_SUBTRACT(loop_timeout, nsock_tod));
     if (msecs_left == 0 && loopnum > 0) {
       quitstatus = NSOCK_LOOP_TIMEOUT;
       break;
@@ -858,6 +1010,8 @@ const struct timeval *nsock_gettimeofday() {
    such as adjusting the descriptor select/poll lists, registering the
    timeout value, etc. */
 void nsp_add_event(mspool *nsp, msevent *nse) {
+  if (nsp->tracelevel > 5)
+      nsock_trace(nsp, "NSE #%lu: Adding event", nse->id);
 
   /* First lets do the event-type independant stuff -- starting with
      timeouts */
@@ -867,7 +1021,7 @@ void nsp_add_event(mspool *nsp, msevent *nse) {
     if (nse->timeout.tv_sec != 0) {
       if (nsp->evl.next_ev.tv_sec == 0) {
 	nsp->evl.next_ev = nse->timeout;
-      } else if (TIMEVAL_NSEC_SUBTRACT(nsp->evl.next_ev, nse->timeout) > 0) {
+      } else if (TIMEVAL_MSEC_SUBTRACT(nsp->evl.next_ev, nse->timeout) > 0) {
 	nsp->evl.next_ev = nse->timeout;
       }
     }
@@ -880,6 +1034,7 @@ void nsp_add_event(mspool *nsp, msevent *nse) {
   case NSE_TYPE_CONNECT:
   case NSE_TYPE_CONNECT_SSL:
     if (!nse->event_done) {
+      assert(nse->iod->sd >= 0);
       FD_SET( nse->iod->sd, &nsp->mioi.fds_master_r);
       FD_SET( nse->iod->sd, &nsp->mioi.fds_master_w);
       FD_SET( nse->iod->sd, &nsp->mioi.fds_master_x);
@@ -890,6 +1045,7 @@ void nsp_add_event(mspool *nsp, msevent *nse) {
 
   case NSE_TYPE_READ:
     if (!nse->event_done) {
+      assert(nse->iod->sd >= 0);
       FD_SET( nse->iod->sd, &nsp->mioi.fds_master_r);
       nsp->mioi.max_sd = MAX(nsp->mioi.max_sd, nse->iod->sd);
 #if HAVE_OPENSSL
@@ -901,6 +1057,7 @@ void nsp_add_event(mspool *nsp, msevent *nse) {
 
   case NSE_TYPE_WRITE:
     if (!nse->event_done) {
+      assert(nse->iod->sd >= 0);
       FD_SET( nse->iod->sd, &nsp->mioi.fds_master_w);
       nsp->mioi.max_sd = MAX(nsp->mioi.max_sd, nse->iod->sd);
 #if HAVE_OPENSSL
@@ -914,6 +1071,36 @@ void nsp_add_event(mspool *nsp, msevent *nse) {
     gh_list_prepend(&nsp->evl.timer_events, nse);
     break;
 
+#if HAVE_PCAP
+  case NSE_TYPE_PCAP_READ:{
+    mspcap *mp = (mspcap *) nse->iod->pcap;
+    assert(mp);
+    if(mp->pcap_desc >= 0){ /* pcap descriptor present */
+      if(!nse->event_done){
+        FD_SET(mp->pcap_desc, &nsp->mioi.fds_master_r);
+        nsp->mioi.max_sd = MAX(nsp->mioi.max_sd, mp->pcap_desc);
+      }
+      if (nsp->tracelevel > 8)
+          nsock_trace(nsp, "PCAP NSE #%lu: Adding event to READ_EVENTS", nse->id);
+      gh_list_prepend(&nsp->evl.read_events, nse);
+      
+      #if PCAP_BSD_SELECT_HACK
+      /* when using BSD hack we must do pcap_next() after select(). 
+       * Let's insert this pcap to bot queues, to selectable and nonselectable.
+       * This will result in doing pcap_next_ex() just before select() */
+      if (nsp->tracelevel > 8)
+          nsock_trace(nsp, "PCAP NSE #%lu: Adding event to PCAP_READ_EVENTS", nse->id);
+      gh_list_prepend(&nsp->evl.pcap_read_events, nse);
+      #endif
+    }else{ /* pcap isn't selectable. Add it to pcap-specific queue. */
+      if (nsp->tracelevel > 8)
+          nsock_trace(nsp, "PCAP NSE #%lu: Adding event to PCAP_READ_EVENTS", nse->id);
+      gh_list_prepend(&nsp->evl.pcap_read_events, nse);
+    }
+    break;
+  }
+#endif    
+
   default:
     assert(0);
     break; /* unreached */
@@ -944,17 +1131,17 @@ void nsock_trace_handler_callback(mspool *ms, msevent *nse) {
   int strlength = 0;
   char displaystr[256];
   char errstr[256];
-
+  
   if (ms->tracelevel == 0)
     return;
 
   nsi = nse->iod;
 
   if (nse->status == NSE_STATUS_ERROR) {
-    snprintf(errstr, sizeof(errstr), "[%s (%d)] ", strerror(nse->errnum), nse->errnum);
+    Snprintf(errstr, sizeof(errstr), "[%s (%d)] ", strerror(nse->errnum), nse->errnum);
   } else errstr[0] = '\0';
 
-  // Some types have special tracing treatment
+  /* Some types have special tracing treatment */
   switch(nse->type) {
   case NSE_TYPE_CONNECT:
   case NSE_TYPE_CONNECT_SSL:
@@ -964,12 +1151,17 @@ void nsock_trace_handler_callback(mspool *ms, msevent *nse) {
     break;
 
   case NSE_TYPE_READ:
-    if (nse->status != NSE_STATUS_SUCCESS)
-      nsock_trace(ms, "Callback: %s %s %sfor EID %li [%s:%hi]", 
+    if (nse->status != NSE_STATUS_SUCCESS) {
+      if (nsi->peerlen > 0)
+	 nsock_trace(ms, "Callback: %s %s %sfor EID %li [%s:%hi]", 
 		  nse_type2str(nse->type), nse_status2str(nse->status), 
 		  errstr, nse->id, inet_ntop_ez(&nsi->peer, nsi->peerlen), 
 		  nsi_peerport(nsi));
-    else {    
+      else
+	 nsock_trace(ms, "Callback: %s %s %sfor EID %li (peer unspecified)",
+		     nse_type2str(nse->type), nse_status2str(nse->status),
+		     errstr, nse->id);
+    } else {    
       str = nse_readbuf(nse, &strlength);
       if (ms->tracelevel > 1 && strlength < 80) {
 	memcpy(displaystr, ": ", 2);
@@ -977,11 +1169,17 @@ void nsock_trace_handler_callback(mspool *ms, msevent *nse) {
 	displaystr[2 + strlength] = '\0';
 	replacenonprintable(displaystr + 2, strlength, '.');
       } else displaystr[0] = '\0';
-      nsock_trace(ms, "Callback: %s %s for EID %li [%s:%hi] %s(%d bytes)%s", 
-		  nse_type2str(nse->type), nse_status2str(nse->status), 
-		  nse->id, inet_ntop_ez(&nsi->peer, nsi->peerlen), 
-		  nsi_peerport(nsi), nse_eof(nse)? "[EOF]" : "", strlength, 
-		  displaystr);
+      
+      if (nsi->peerlen > 0)
+	nsock_trace(ms, "Callback: %s %s for EID %li [%s:%hi] %s(%d bytes)%s", 
+      		    nse_type2str(nse->type), nse_status2str(nse->status), 
+      		    nse->id, inet_ntop_ez(&nsi->peer, nsi->peerlen), 
+      		    nsi_peerport(nsi), nse_eof(nse)? "[EOF]" : "", strlength, 
+      		    displaystr);
+      else
+	nsock_trace(ms, "Callback %s %s for EID %li (peer unspecified) %s(%d bytes)%s",
+		    nse_type2str(nse->type), nse_status2str(nse->status),
+		    nse->id, nse_eof(nse)? "[EOF]" : "", strlength, displaystr);
     }
     break;
 
@@ -995,6 +1193,17 @@ void nsock_trace_handler_callback(mspool *ms, msevent *nse) {
     nsock_trace(ms, "Callback: %s %s %sfor EID %li", 
 		nse_type2str(nse->type), nse_status2str(nse->status), errstr, 
 		nse->id);
+	
+    break;	
+#if HAVE_PCAP
+  case NSE_TYPE_PCAP_READ:
+	 nsock_trace(ms, "Callback: %s %s %sfor EID %li ",
+		     nse_type2str(nse->type), nse_status2str(nse->status),
+		     errstr, nse->id);
+    break;
+#endif
+   default:
+   assert(0);
     break;
   }
 
