@@ -1,6 +1,6 @@
 
 /***************************************************************************
- * output.c -- Handles the Nmap output system.  This currently involves    *
+ * output.h -- Handles the Nmap output system.  This currently involves    *
  * console-style human readable output, XML output, Script |<iddi3         *
  * output, and the legacy greppable output (used to be called "machine     *
  * readable").  I expect that future output forms (such as HTML) may be    *
@@ -9,17 +9,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -28,7 +28,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -42,7 +42,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://nmap.org to download Nmap.                                       *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -54,16 +54,16 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -81,7 +81,7 @@
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
+ * distribution.  By sending these changes to Fyodor or one of the         *
  * Insecure.Org development mailing lists, it is assumed that you are      *
  * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
  * to reuse, modify, and relicense the code.  Nmap will always be          *
@@ -95,31 +95,34 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: output.h 3024 2005-12-31 00:32:42Z fyodor $ */
+/* $Id: output.h 7640 2008-05-22 20:45:32Z fyodor $ */
 
 #ifndef OUTPUT_H
 #define OUTPUT_H
 
-#define LOG_TYPES 5
-#define LOG_MASK 31
+#define LOG_NUM_FILES 4 /* # of values that actual files (they must come first */
+#define LOG_FILE_MASK 15 /* The mask for log typs in the file array */
 #define LOG_NORMAL 1
 #define LOG_MACHINE 2
-#define LOG_HTML 4
-#define LOG_SKID 8
-#define LOG_XML 16
+#define LOG_SKID 4
+#define LOG_XML 8
 #define LOG_STDOUT 1024
-#define LOG_SKID_NOXLT 2048
+#define LOG_STDERR 2048
+#define LOG_SKID_NOXLT 4096
+#define LOG_MAX LOG_SKID_NOXLT /* The maximum log type value */
 
-#define LOG_NAMES {"normal", "machine", "HTML", "$Cr!pT |<!dd!3", "XML"}
+#define LOG_PLAIN LOG_NORMAL|LOG_SKID|LOG_STDOUT
+
+#define LOG_NAMES {"normal", "machine", "$Cr!pT |<!dd!3", "XML"}
 
 #include "portlist.h"
-#include "tcpip.h"
+#include "nmap.h"
 #include "global_structures.h"
 
 /* Prints the familiar Nmap tabular output showing the "interesting"
@@ -135,14 +138,21 @@ void printportoutput(Target *currenths, PortList *plist);
    in a certain place to conform to DTD. */
 void printmacinfo(Target *currenths);
 
-/* Prints the MAC address (if discovered) to XML output */
-void print_MAC_XML_Info(Target *currenths);
+char *logfilename(const char *str, struct tm *tm);
 
 /* Write some information (printf style args) to the given log stream(s).
    Remember to watch out for format string bugs. */
 void log_write(int logt, const char *fmt, ...)
      __attribute__ ((format (printf, 2, 3)));
 
+/* This is the workhorse of the logging functions.  Usually it is
+   called through log_write(), but it can be called directly if you
+   are dealing with a vfprintf-style va_list.  Unlike log_write, YOU
+   CAN ONLY CALL THIS WITH ONE LOG TYPE (not a bitmask full of them).
+   In addition, YOU MUST SANDWHICH EACH EXECUTION IF THIS CALL BETWEEN
+   va_start() AND va_end() calls. */
+void log_vwrite(int logt, const char *fmt, va_list ap);
+
 /* Close the given log stream(s) */
 void log_close(int logt);
 
@@ -159,9 +169,6 @@ void log_flush_all();
    it already exists.  If the file does not exist, it will be created */
 int log_open(int logt, int append, char *filename);
 
-/* Used in creating skript kiddie style output.  |<-R4d! */
-void skid_output(char *s);
-
 /* Output the list of ports scanned to the top of machine parseable
    logs (in a comment, unfortunately).  The items in ports should be
    in sequential order for space savings and easier to read output */
@@ -169,13 +176,6 @@ void output_ports_to_machine_parseable_output(struct scan_lists *ports,
 					      int tcpscan, int udpscan,
 					      int protscan);
 
-/* The items in ports should be
-   in sequential order for space savings and easier to read output.  Outputs
-   the rangelist to the log stream given (such as LOG_MACHINE or LOG_XML) */
-void output_rangelist_given_ports(int logt, unsigned short *ports,
-				  int numports);
-
-
 /* Similar to output_ports_to_machine_parseable_output, this function
    outputs the XML version, which is scaninfo records of each scan
    requested and the ports which it will scan for */
@@ -195,6 +195,9 @@ void printosscanoutput(Target *currenths);
    service scan (if it was performed) */
 void printserviceinfooutput(Target *currenths);
 
+void printhostscriptresults(Target *currenths);
+char* formatScriptOutput(struct script_scan_result ssr);
+
 /* Print a detailed list of Nmap interfaces and routes to
    normal/skiddy/stdout output */
 int print_iflist(void);
@@ -206,5 +209,9 @@ void printStatusMessage();
    of an Nmap run */
 void printfinaloutput();
 
+/* Prints the names of data files that were loaded and the paths at which they
+   were found. */
+void printdatafilepaths();
+
 char* xml_convert (const char* str);
 #endif /* OUTPUT_H */
